import collections
from contextlib import contextmanager
import os
import time

import jwt
from pypac import PACSession
from requests.auth import HTTPProxyAuth

from ayx_plugin_sdk.core.exceptions import WorkflowRuntimeError
import AlteryxPythonSDK as Sdk


ADOBE_ANALYTICS_METASCOPE = (
    "https://ims-na1.adobelogin.com/s/ent_analytics_bulk_ingest_sdk"
)
JwtConfiguration = collections.namedtuple(
    "JwtConfiguration", "exp_in, iss, sub, aud, private_key"
)


def create_jwt_payload(config: JwtConfiguration) -> object:
    """
    helper function to create a jwt payload from a defined config
    """
    return {
        "exp": round(time.time() + config.exp_in),
        "iss": config.iss,
        "sub": config.sub,
        "aud": config.aud,
        ADOBE_ANALYTICS_METASCOPE: True,
    }


def get_jwt(config: JwtConfiguration) -> str:
    """
    helper function to create a base64 encoded jwt
    """
    return jwt.encode(
        payload=create_jwt_payload(config), key=config.private_key, algorithm="RS256"
    )


def create_request_payload(client_id: str, client_secret: str, jwt_str: str) -> dict:
    """
    helper function to create the access token request payload
    """
    return {
        "client_id": client_id,
        "client_secret": client_secret,
        "jwt_token": jwt_str,
    }


def request_access_token(error_message: str, payload: dict):
    """
    request an access token from Adobe Analytis
    """
    url = "https://ims-na1.adobelogin.com/ims/exchange/jwt"

    session = create_pac_session(url)

    response = access_token_session_helper(session,url,payload)

    if response.status_code != 200:
        raise WorkflowRuntimeError(
            error_message + str(response.status_code) + ", " + response.text
        )

    return response.json()

def access_token_session_helper(session, url: str, payload: dict):
    """
    helper fun to request an access token from Adobe Analytis in proxy
    environment. Facilitates some level of unit testing
    """
    with proxy_context_for_url(session, url):

        header_jwt = {
            "cache-control": "no-cache",
            "content-type": "application/x-www-form-urlencoded",
        }

        response = session.post(
            url,
            headers=header_jwt,
            data=payload,
        )
    
    return response


def parse_access_token_resp(body: dict) -> str:
    """
    parse the access token from the Adobe Analytics response
    """
    return body["access_token"]


def create_pac_session(url=None):
    """
    Simplifies creating a PACSession object with authentication set up from Alteryx settings.

    Returns a PACSession object with authentication set up if required.
    """
    proxy_info = Sdk.ProxyInfo()  # pylint: disable=E1101
    session = PACSession()
    if proxy_info.auth_required:
        session.proxy_auth = HTTPProxyAuth(proxy_info.username, proxy_info.password)
    if url:
        pac = session.get_pac()
        if pac:
            # try to access the URL so it goes through the proxy failover process
            # banned/failed proxies are stored in the resolver inside the session
            try:
                session.head(url)
            except Exception: # pylint: disable=W0703
                 # ignore any exceptions here
                pass
    return session


@contextmanager
def proxy_context_for_url(session, url):
    """
    This context manager provides a simple way to add rudimentary PAC functionality
    to code that cannot be modified to use :class:`PACSession`,
    but obeys the ``HTTP_PROXY`` and ``HTTPS_PROXY`` environment variables.

    Upon entering this context, PAC discovery occurs with default parameters.
    If a PAC is found, then it's asked for the proxy to use for the given URL.
    The proxy environment variables are then set accordingly.

    If there is no pac available, the manual proxy from the Windows settings is used.
    If there is no proxy configured, then the environment varaibles are set to empty strings.

    :param url: Consult the PAC or bypass list for the proxy to use for this URL.
    """
    prev_http_proxy, prev_https_proxy = (
        os.environ.get("HTTP_PROXY"),
        os.environ.get("HTTPS_PROXY"),
    )
    pac = session.get_pac()
    if pac:
        # this might break in later versions of PyPAC because it is using
        # an undocumented data member of PACSession. It is necessary because
        # banned/failed proxy information is stored in the resolver, so if we
        # create a new resolver, we'd lose all of it, and we'd be unable to deal with
        # any stored bans from failover when we created the session.
        resolver = session._proxy_resolver # pylint: disable=W0212
        proxies = resolver.get_proxy_for_requests(url)
        os.environ["HTTP_PROXY"] = proxies.get("http") or ""
        os.environ["HTTPS_PROXY"] = proxies.get("https") or ""
    else:
        proxy_info = Sdk.ProxyInfo() # pylint: disable=E1101
        proxy = proxy_info.get_manual_proxy(url)
        if proxy:
            os.environ["HTTP_PROXY"] = proxy
            os.environ["HTTPS_PROXY"] = proxy
        else:
            os.environ["HTTP_PROXY"] = ""
            os.environ["HTTPS_PROXY"] = ""
    yield
    if prev_http_proxy:
        os.environ["HTTP_PROXY"] = prev_http_proxy
    elif "HTTP_PROXY" in os.environ:
        del os.environ["HTTP_PROXY"]
    if prev_https_proxy:
        os.environ["HTTPS_PROXY"] = prev_https_proxy
    elif "HTTPS_PROXY" in os.environ:
        del os.environ["HTTPS_PROXY"]
