# Copyright (C) 2020 Alteryx, Inc. All rights reserved.
#
# Licensed under the Apache License, Version 2.0 (the "License");
# you may not use this file except in compliance with the License.
# You may obtain a copy of the License at
#
#    http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS,
# WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
# See the License for the specific language governing permissions and
# limitations under the License.
"""Example input tool."""
from ayx_plugin_sdk.core import (
    FieldType,
    InputConnectionBase,
    Metadata,
    Plugin,
    ProviderBase,
    RecordPacket,
    register_plugin,
)
import jwt_utilities as utils

import pandas as pd

ACCESS_TOKEN_FIELD_NAME = "access_token"
ACCESS_TOKEN_LEN = 4000  # should be less than this due to limits on max url length

# lint warning suppressions

# Plugin init method will raise if called
# pylint: disable=W0231

# tool name follows its naming convetion to facilitate versioning
# pylint: disable=C0103


class Adobe_Service_Authentication_v1_0_0(Plugin):

    """Concrete implementation of an AyxPlugin."""

    def __init__(self, provider: ProviderBase) -> None:
        """Construct a plugin."""
        self.provider = provider
        self.tool_config = provider.tool_config

        self.jwt_configuration = utils.JwtConfiguration(
            exp_in=int(self.tool_config["exp_in"]) if self.tool_config["exp_in"] else 0,
            iss=self.tool_config["iss"],
            sub=self.tool_config["sub"],
            aud=self.tool_config["aud"],
            private_key=self.tool_config["private_key"],
        )

        self.client_id = self.tool_config["client_id"]
        self.client_secret = self.tool_config["client_secret"]

        self.output_anchor = self.provider.get_output_anchor("Output")

        self.output_metadata = Metadata()
        self.output_metadata.add_field(
            ACCESS_TOKEN_FIELD_NAME, FieldType.v_string, size=ACCESS_TOKEN_LEN
        )

        self.output_anchor.open(self.output_metadata)

        self.provider.logger.debug("Plugin initialized.")

    def on_input_connection_opened(self, input_connection: InputConnectionBase) -> None:
        """Initialize the Input Connections of this plugin."""
        raise NotImplementedError("Input tools don't have input connections.")

    def on_record_packet(self, input_connection: InputConnectionBase) -> None:
        """Handle the record packet received through the input connection."""

    def on_complete(self) -> None:
        """Create all records."""

        jwt = utils.get_jwt(self.jwt_configuration)
        payload = utils.create_request_payload(self.client_id, self.client_secret, jwt)
        access_token_resp = utils.request_access_token(self.provider.io.translate_msg("Authentication has failed: "), payload)
        access_token = utils.parse_access_token_resp(access_token_resp)

        data_frame = pd.DataFrame({ACCESS_TOKEN_FIELD_NAME: [access_token]})
        packet = RecordPacket.from_dataframe(self.output_metadata, data_frame)

        self.output_anchor.write(packet)
        self.provider.logger.debug("Successfully recieved access token")


AyxPlugin = register_plugin(Adobe_Service_Authentication_v1_0_0)
